@@ -1,12 +1,9 @@
 import click
 import collections
-import os
 import signal
 import sys
-import textwrap
-from octofludb.util import log
-import octofludb.classes as classes
-import octofludb.script as script
+import os
+from octofludb.util import log, die
 
 
 def open_graph():
@@ -30,7 +27,10 @@ def with_graph(f, *args, filename=None, outfile=sys.stdout, **kwargs):
     turtles = g.serialize(format="turtle")
     log("done")
     for l in turtles.splitlines():
-        print(l.decode("utf-8"), file=outfile)
+        try:
+          print(l.decode("utf-8"), file=outfile)
+        except:
+          print(l, file=outfile)
     g.close()
 
 
@@ -121,6 +121,7 @@ def init_cmd(url, repo):
     import pgraphdb as db
     import requests
     import shutil
+    import octofludb.script as script
 
     config_file = os.path.join(
         os.path.dirname(__file__), "data", "octofludb-config.ttl"
@@ -150,43 +151,8 @@ def init_cmd(url, repo):
     script.initialize_config_file()
 
 
-@click.command(
-    name="pull",
-)
-@click.option(
-    "--nmonths",
-    help="Update Genbank files for the last N months",
-    default=3,
-    type=click.IntRange(min=1, max=9999),
-)
-@url_opt
-@repo_name_opt
-def pull_cmd(nmonths, url, repo):
-    """
-    Update data. Pull from genbank, process any new data in the data folder,
-    assign clades to swine data, assign subtypes to all data, and extract
-    motifs
-    """
-    import octofludb.recipes as recipe
-    import pgraphdb as db
-
-    cwd = os.getcwd()
-
-    script.gotoBuildHome()
-
-    config = script.load_config_file()
-
-    # upload ontological schema
-    schema_file = script.get_data_file("schema.ttl")  #
-    upload([schema_file], url=url, repo=repo)
-
-    # upload geological relationships
-    geog_file = upload([script.get_data_file("geography.ttl")], url=url, repo=repo)[0]
-
-    # update genbank (take a parameter telling how far back to go)
-    # this command fills the current directory with .gb* files
-    gb_turtles = prep_update_gb(minyear=1900, maxyear=2121, nmonths=nmonths)
-    upload(gb_turtles, url=url, repo=repo)
+def upload_gisaid(config, url, repo):
+    import octofludb.script as script
 
     epiflu_metafiles = script.epiflu_meta_files(config)
     skipped_meta = 0
@@ -225,6 +191,11 @@ def pull_cmd(nmonths, url, repo):
             f"Skipped {str(skipped_fasta)} epiflu fasta files where existing non-empty turtle files were found in the build directory"
         )
 
+
+def upload_classifications(config, url, repo):
+    import octofludb.script as script
+    import pgraphdb as db
+
     # octoflu classifications of unclassified swine
     # * retrieve unclassified strains
     unclassified_fasta = "unclassified-swine.fna"
@@ -252,6 +223,26 @@ def pull_cmd(nmonths, url, repo):
 
     upload([unclassified_turtle], url=url, repo=repo)
 
+    # infer constellations
+    constellation_table = "constellations.txt"
+    constellation_turtles = "constellations.ttl"
+
+    delete_constellations = script.get_data_file("delete-constellations.rq")
+    db.update(sparql_file=delete_constellations, url=url, repo_name=repo)
+
+    with open(constellation_table, "w") as constout:
+        make_const(url=url, repo=repo, outfile=constout)
+
+    with open(constellation_turtles, "w") as turtleout:
+        prep_table(constellation_table, outfile=turtleout)
+
+    upload([constellation_turtles], url=url, repo=repo)
+
+
+def upload_subtypes(config, url, repo):
+    import octofludb.script as script
+    import pgraphdb as db
+
     # infer subtypes
     subtypes_table = "subtypes.txt"
     with open(subtypes_table, "w") as subtypesout:
@@ -281,20 +272,41 @@ def pull_cmd(nmonths, url, repo):
         prep_table(epiflu_subtypes, outfile=eturtleout)
     upload([gturtles, eturtles], url=url, repo=repo)
 
-    # infer constellations
-    constellation_table = "constellations.txt"
-    constellation_turtles = "constellations.ttl"
 
-    delete_constellations = script.get_data_file("delete-constellations.rq")
-    db.update(sparql_file=delete_constellations, url=url, repo_name=repo)
+def upload_motifs(config, url, repo):
+    import octofludb.script as script
 
-    with open(constellation_table, "w") as constout:
-        make_const(url=url, repo=repo, outfile=constout)
+    # find h1 motifs
+    h1_motif_table = script.findMotifs(
+        os.path.join(os.path.dirname(__file__), "data", "get-h1-swine.rq"),
+        [
+            "sa_motif=124,125,155,157,159,160,162,163,164",
+            "sb_motif=153,156,189,190,193,195",
+            "ca1_motif=166,170,204,237",
+            "ca2_motif=137,140,142,221,222",
+            "cb_motif=70,71,73,74,75,115",
+        ],
+        "H1",
+        url=url,
+        repo_name=repo,
+    )
 
-    with open(constellation_turtles, "w") as turtleout:
-        prep_table(constellation_table, outfile=turtleout)
+    with open("h1-motifs.ttl", "w") as turtleout:
+        prep_table(h1_motif_table, outfile=turtleout)
+    upload(["h1-motifs.ttl"], url=url, repo=repo)
 
-    upload([constellation_turtles], url=url, repo=repo)
+    # find h3 motifs
+    h3_motif_table = script.findMotifs(
+        os.path.join(os.path.dirname(__file__), "data", "get-h3-swine.rq"),
+        ["h3_motif=145,155,156,158,189"],
+        "H3",
+        url=url,
+        repo_name=repo,
+    )
+
+    with open("h3-motifs.ttl", "w") as turtleout:
+        prep_table(h3_motif_table, outfile=turtleout)
+    upload(["h3-motifs.ttl"], url=url, repo=repo)
 
     # load all tags
     for (tag, basename) in config["tags"].items():
@@ -304,21 +316,92 @@ def pull_cmd(nmonths, url, repo):
                 prep_tag(tag, filename, outfile=f)
             upload([outfile], url=url, repo=repo)
 
-    os.chdir(cwd)
-
-    return None
-
 
 @click.command(
-    name="clean",
+    name="pull",
+)
+@click.option(
+    "--nmonths",
+    help="Update Genbank files for the last N months",
+    default=1,
+    type=click.IntRange(min=0, max=9999),
+)
+@click.option(
+    "--no-schema", is_flag=True, default=False, help="Skip upload schema steps"
+)
+@click.option(
+    "--include-gisaid", is_flag=True, default=False, help="include gisaid data step"
+)
+@click.option(
+    "--no-clades",
+    is_flag=True,
+    default=False,
+    help="Skip clade and constellation classification steps",
+)
+@click.option(
+    "--no-subtype",
+    is_flag=True,
+    default=False,
+    help="Skip subtype inferrence step",
+)
+@click.option(
+    "--no-motifs", is_flag=True, default=False, help="Skip motif extraction steps"
 )
 @url_opt
 @repo_name_opt
-def clean_cmd(url, repo):
+def pull_cmd(
+    nmonths, no_schema, include_gisaid, no_clades, no_subtype, no_motifs, url, repo
+):
     """
-    clean the database (not yet implemented)
+    Update data. Pull from genbank, process any new data in the data folder,
+    assign clades to swine data, assign subtypes to all data, and extract
+    motifs.
+
+    To build the database from nothing call `octofludb pull --nmonths=360`.
+    This will pull all genbank data that has been released in the last 30
+    years (which should be all of it).
     """
-    raise NotImplemented
+    import octofludb.recipes as recipe
+    import octofludb.script as script
+    import pgraphdb as db
+
+    cwd = os.getcwd()
+
+    script.gotoBuildHome()
+
+    config = script.load_config_file()
+
+    if not no_schema:
+        # upload ontological schema
+        schema_file = script.get_data_file("schema.ttl")  #
+        upload([schema_file], url=url, repo=repo)
+
+        # upload geological relationships
+        geog_file = upload([script.get_data_file("geography.ttl")], url=url, repo=repo)[
+            0
+        ]
+
+    if nmonths > 0:
+        # update genbank (take a parameter telling how far back to go)
+        # this command fills the current directory with .gb* files
+        gb_turtles = prep_update_gb(minyear=1900, maxyear=2121, nmonths=nmonths)
+        upload(gb_turtles, url=url, repo=repo)
+
+    if include_gisaid:
+        upload_gisaid(config, url, repo)
+
+    if not no_clades:
+        upload_classifications(config, url, repo)
+
+    if not no_subtype:
+        upload_subtypes(config, url, repo)
+
+    if not no_motifs:
+        upload_motifs(config, url, repo)
+
+    os.chdir(cwd)
+
+    return None
 
 
 def fmt_query_cmd(sparql_filename, header, fasta, url, repo, outfile=sys.stdout):
@@ -370,16 +453,21 @@ def classify_cmd(filename, reference=None):
 
 
 def classify_and_write(filename, reference=None, outfile=sys.stdout):
+    import octofludb.colors as colors
+
     rows = classify(filename, reference=reference)
-    if rows:
-        print("strain\tsegment\tus_clade\tglobal_clade", file=outfile)
-        for row in rows:
-            print("\t".join(row), file=outfile)
-    else:
-        die(colors.bad("No result from running octFLU"), file=outfile)
+    print("seqid\tsegment_subtype\tclade\tgl_clade", file=outfile)
+
+    # This may be empty, that is fine. A table with only a heade line would
+    # generate an empty turtle file. Uploading an empty turtle file does
+    # nothing.
+    for row in rows:
+        print("\t".join(row), file=outfile)
 
 
 def classify(filename, reference=None):
+    import octofludb.script as script
+
     if not reference:
         config = script.load_config_file()
         reference = script.get_octoflu_reference(config)
@@ -436,6 +524,7 @@ def upload_cmd(turtle_filenames, url, repo):
 
 def upload(turtle_filenames, url, repo):
     import pgraphdb as db
+    import octofludb.script as script
 
     files = []
     for filenames in turtle_filenames:
@@ -454,11 +543,11 @@ def upload(turtle_filenames, url, repo):
 )
 @click.argument("tag", type=str)
 @filename_arg
-def prep_tag_cmd(tag, filename, outfile=sys.stdout):
+def prep_tag_cmd(tag, filename):
     """
     Associate list of IDs with a tag
     """
-    prep_tag(tag, filename, outfile=outfile)
+    prep_tag(tag, filename)
 
 
 def prep_tag(tag, filename, outfile=sys.stdout):
@@ -481,7 +570,10 @@ def prep_tag(tag, filename, outfile=sys.stdout):
         turtles = g.serialize(format="turtle")
         log("done")
         for l in turtles.splitlines():
-            print(l.decode("utf-8"), file=outfile)
+            try:
+                print(l.decode("utf-8"), file=outfile)
+            except:
+                print(l, file=outfile)
     g.close()
 
 
@@ -680,6 +772,7 @@ def prep_table(
     Translate a table to RDF
     """
     from octofludb.recipes import IrregularSegmentTable
+    import octofludb.classes as classes
 
     if segment_key:
         constructor = IrregularSegmentTable
@@ -729,6 +822,8 @@ def prep_fasta(
     na=None,
     outfile=sys.stdout,
 ):
+    import octofludb.classes as classes
+
     def _mk_fasta_cmd(g, fh):
         (inc, exc, levels) = process_tablelike(include, exclude, None)
         classes.Ragged(
@@ -741,7 +836,8 @@ def prep_fasta(
             na_str=make_na(na),
         ).connect(g)
 
-    with_graph(_mk_fasta_cmd, filename, outfile=outfile)
+    with open(filename, "r") as fasta_fh:
+        with_graph(_mk_fasta_cmd, fasta_fh, outfile=outfile)
 
     return outfile
 
@@ -824,15 +920,7 @@ prep_grp.add_command(prep_ird_cmd)
 prep_grp.add_command(prep_ivr_cmd)
 
 
-# ===== make subcommands ====
-
-
-@click.command(
-    name="const",
-)
-@url_opt
-@repo_name_opt
-def make_const_cmd(url, repo):
+def make_const(url, repo, outfile=sys.stdout):
     """
     Generate constellations for all swine strains.
 
@@ -845,10 +933,6 @@ def make_const_cmd(url, repo):
     only a few of these in the US. "X" represents something else exotic (e.g.,
     not US). For mixed strains, the constellation will be recorded as "mixed".
     """
-    make_const(url, repo)
-
-
-def make_const(url, repo, outfile=sys.stdout):
     import octofludb.formatting as formatting
     import pgraphdb as db
 
@@ -857,18 +941,6 @@ def make_const(url, repo, outfile=sys.stdout):
     formatting.write_constellations(results, outfile=outfile)
 
 
-@click.command(
-    name="subtypes",
-)
-@url_opt
-@repo_name_opt
-def make_subtypes_cmd(url, repo, outfile=sys.stdout):
-    """
-    Determine subtypes based on Genbank serotype field, epiflu data, or octoflu HA/NA classifications"
-    """
-    make_subtypes(url=url, repo=repo, outfile=outfile)
-
-
 def make_subtypes(url, repo, outfile=sys.stdout):
     strains, isolates = get_missing_subtypes(url, repo)
     print("strain_name\tsubtype", file=outfile)
@@ -892,7 +964,7 @@ def get_missing_subtypes(url, repo):
 )
 @url_opt
 @repo_name_opt
-def make_masterlist_cmd(url, repo):
+def report_masterlist_cmd(url, repo):
     """
     Generate the surveillance masterlist
     """
@@ -906,23 +978,6 @@ def make_masterlist_cmd(url, repo):
     recipe.mk_masterlist(results)
 
 
-@click.group(
-    cls=OrderedGroup,
-    name="make",
-    context_settings=CONTEXT_SETTINGS,
-)
-def make_grp():
-    """
-    Derive constellations, subtypes and such
-    """
-    pass
-
-
-make_grp.add_command(make_const_cmd)
-make_grp.add_command(make_subtypes_cmd)
-make_grp.add_command(make_masterlist_cmd)
-
-
 # ===== fetch subcommands =====
 
 
@@ -957,7 +1012,10 @@ def fetch_tag_cmd(filename, url, repo):
     (n, turtle_filename) = mkstemp(suffix=".ttl")
     with open(turtle_filename, "w") as th:
         for l in turtles.splitlines():
-            print(l.decode("utf-8"), file=th)
+            try:
+                print(l.decode("utf-8"), file=th)
+            except:
+                print(l, file=th)
 
     # upload it to the database
     upload_cmd([turtle_filename], url, repo)
@@ -1093,9 +1151,9 @@ def macro_query(filename, macros, *args, **kwargs):
                     line = re.sub(macro, replacement, line)
                 print(line, file=query)
 
-    fmt_query_cmd(sparql_filename, *args, **kwargs)
+    fmt_query_cmd(tmpfile, *args, **kwargs)
 
-    os.remove(tempfile)
+    os.remove(tmpfile)
 
 
 @click.command(
@@ -1147,7 +1205,7 @@ def report_quarter_cmd(url, repo):
     Currently, this just generates the smae masterlist as is used in
     octoflushow. However, it may eventually be specialized.
     """
-    make_masterlist_cmd(url=url, repo=repo)
+    report_masterlist_cmd(url=url, repo=repo)
 
 
 @click.command(
@@ -1174,10 +1232,10 @@ def report_grp():
     pass
 
 
+report_grp.add_command(report_offlu_cmd)
 report_grp.add_command(report_monthly_cmd)
 report_grp.add_command(report_quarter_cmd)
-report_grp.add_command(report_offlu_cmd)
-
+report_grp.add_command(report_masterlist_cmd)
 
 # ===== deletion subcommands =====
 
@@ -1191,7 +1249,11 @@ def delete_constellations_cmd(url, repo):
     """
     Delete all constellation data
     """
-    raise NotImplemented
+    import octofludb.script as script
+    import pgraphdb as db
+
+    delete_script = script.get_data_file("delete-constellations.rq")
+    db.update(sparql_file=delete_script, url=url, repo_name=repo)
 
 
 @click.command(
@@ -1203,23 +1265,31 @@ def delete_subtypes_cmd(url, repo):
     """
     Delete all subtype data
     """
-    raise NotImplemented
+    import octofludb.script as script
+    import pgraphdb as db
+
+    delete_script = script.get_data_file("delete-subtypes.rq")
+    db.update(sparql_file=delete_script, url=url, repo_name=repo)
 
 
 @click.command(
-    name="clades",
+    name="us-clades",
 )
 @url_opt
 @repo_name_opt
-def delete_clades_cmd(url, repo):
+def delete_us_clades_cmd(url, repo):
     """
     Delete all clade data
     """
-    raise NotImplemented
+    import octofludb.script as script
+    import pgraphdb as db
+
+    delete_script = script.get_data_file("delete-us_clades.rq")
+    db.update(sparql_file=delete_script, url=url, repo_name=repo)
 
 
 @click.command(
-    name="gl_clades",
+    name="gl-clades",
 )
 @url_opt
 @repo_name_opt
@@ -1227,7 +1297,27 @@ def delete_gl_clades_cmd(url, repo):
     """
     Delete all global H1 clade data
     """
-    raise NotImplemented
+    import octofludb.script as script
+    import pgraphdb as db
+
+    delete_script = script.get_data_file("delete-gl_clades.rq")
+    db.update(sparql_file=delete_script, url=url, repo_name=repo)
+
+
+@click.command(
+    name="motifs",
+)
+@url_opt
+@repo_name_opt
+def delete_motifs_cmd(url, repo):
+    """
+    Delete all antigenic motifs
+    """
+    import octofludb.script as script
+    import pgraphdb as db
+
+    delete_script = script.get_data_file("delete-motifs.rq")
+    db.update(sparql_file=delete_script, url=url, repo_name=repo)
 
 
 @click.group(
@@ -1237,15 +1327,22 @@ def delete_gl_clades_cmd(url, repo):
 )
 def delete_grp():
     """
-    Remove data from the database in various ways
+    Delete datasets. This can be useful when the dataset is corrupted in some
+    way. For example, if you changed clade names in your octoFLU reference file
+    and then reran `octofludb pull`, no change would occur since only strains
+    with missing clades are updated. Instead you would need to first delete the
+    existing clades and restart. If instead you manually ran octoFLU, built a
+    turtle file from the resulting file, and uploaded that, then you could end
+    up with multiple clades associated with some sequences.
     """
     pass
 
 
 delete_grp.add_command(delete_constellations_cmd)
 delete_grp.add_command(delete_subtypes_cmd)
-delete_grp.add_command(delete_clades_cmd)
+delete_grp.add_command(delete_us_clades_cmd)
 delete_grp.add_command(delete_gl_clades_cmd)
+delete_grp.add_command(delete_motifs_cmd)
 
 
 @click.group(cls=OrderedGroup, context_settings=CONTEXT_SETTINGS)
@@ -1258,14 +1355,12 @@ def cli_grp():
 
 cli_grp.add_command(init_cmd)
 cli_grp.add_command(pull_cmd)
-cli_grp.add_command(clean_cmd)
 cli_grp.add_command(query_cmd)
 cli_grp.add_command(update_cmd)
 cli_grp.add_command(classify_cmd)
 cli_grp.add_command(construct_cmd)
 cli_grp.add_command(upload_cmd)
 cli_grp.add_command(prep_grp)
-cli_grp.add_command(make_grp)
 cli_grp.add_command(fetch_grp)
 cli_grp.add_command(report_grp)
 cli_grp.add_command(delete_grp)
